/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_DOCK_H__
#define __KIBA_DOCK_H__

#include <glib.h>
#include <gtk/gtk.h>

#include <kiba-drawable-object.h>
#include <kiba-object.h>
#include <kiba-cairo-utils.h>
#include <kiba-types.h>
#include <kiba-macros.h>

G_BEGIN_DECLS

#define KIBA_TYPE_DOCK           (kiba_dock_get_type ())
#define KIBA_DOCK(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DOCK, KibaDock))
#define KIBA_DOCK_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_DOCK,  KibaDockClass))
#define KIBA_IS_DOCK(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_IS_DOCK_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_DOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DOCK, KibaDockClass))

/**
 * Property macros.
 **/
/**
 * KIBA_DOCK_IS_HIDDEN:
 * @dock: The KibaDock Instance.
 *
 * Returns wether the dock is hidden.
 *
 * Returns: Wether the dock is hidden.
 */
#define KIBA_DOCK_IS_HIDDEN(dock)    (KIBA_DOCK(dock)->autohide.state == DOCK_HIDDEN)
/**
 * KIBA_DOCK_IS_UNHIDDEN:
 * @dock: The KibaDock Instance.
 *
 * Returns wether the dock is unhidden.
 *
 * Returns: Wether the dock is unhidden.
 */
#define KIBA_DOCK_IS_UNHIDDEN(dock)  (KIBA_DOCK(dock)->autohide.state == DOCK_UNHIDDEN)
/**
 * KIBA_DOCK_IS_DRAWABLE:
 * @dock: The KibaDock Instance.
 *
 * Returns wether the dock is drawable.
 *
 * Returns: Wether the dock is drawable.
 */
#define KIBA_DOCK_IS_DRAWABLE(dock)  GET_FLAG(KIBA_DOCK(dock)->flags, KIBA_DOCK_DRAWABLE)

typedef struct _KibaDock        KibaDock;
typedef struct _KibaDockClass   KibaDockClass;
typedef struct _KibaDockLayout  KibaDockLayout;
typedef struct _KibaDockShape   KibaDockShape;

/**
 * KibaGroupSortFunc:
 * @Param1: The KibaDock Instance, associated to the group.
 * @Param2: The Group.
 * @Param3: A unsorted List of all Objects from group.
 * @Param4: User Data.
 *
 * Callback function to sort all Objects from the same group
 *
 * Returns: the sorted GList.
 */
typedef GList* (*KibaGroupSortFunc) (KibaDock*, char*, const GList*, gpointer);

/**
 * KibaDockPopulateFunc:
 * @Param1: The KibaDock Instance.
 * @Param2: User Data.
 *
 * Method to add Objects on the dock.
 */
typedef void (*KibaDockPopulateFunc) (KibaDock*, gpointer);

/**
 * KibaDockShapeDrawFunc:
 * @Param1: The KibaDock Instance.
 * @Param2: The drawing context.
 * @Param3: User Data.
 *
 * Function to render the dock shape.
 */
typedef void (*KibaDockShapeDrawFunc) (KibaDock*, cairo_t*, gpointer);

/**
 * KibaDockSeparatorDrawFunc:
 * @Param1: The KibaDock Instance.
 * @Param2: The drawing context.
 * @Param3: User Data.
 *
 * Function to render the seperator objects.
 */
typedef void (*KibaDockSeparatorDrawFunc) (KibaDock*, cairo_t*, gpointer);

/**
 * KibaDockClipFunc:
 * @Param1: The KibaDock Instance.
 * @Param2: The drawing context.
 * @Param3: User Data.
 *
 * ...
 */
typedef void (*KibaDockClipFunc) (KibaDock*, cairo_t*, gpointer);

/**
 * KibaDockChildPositionFunc:
 * @Param1: A KibaDock Instance.
 * @Param2: User Data.
 *
 * Custom method to place the objects on the dock.
 */
typedef void (*KibaDockChildPositionFunc) (KibaDock*, gpointer);

/**
 * Dock flags.
 **/
enum {
	KIBA_DOCK_AUTOHIDE = 0,
	KIBA_DOCK_DRAW_SEPARATOR,
	KIBA_DOCK_SET_STRUT,
	KIBA_DOCK_DRAW_CURVES,
	KIBA_DOCK_DRAW_SHADOW,
	KIBA_DOCK_DRAW_PRELIGHT,
	KIBA_DOCK_DRAW_REFLECTIONS,
	KIBA_DOCK_DRAW_PROGRESS,
	KIBA_DOCK_DRAWABLE
};

/**
 * Dock layout type.
 **/
struct _KibaDockLayout {
	KibaPosition  position;       /**< The dock position. You can get the layout data like this:
					   const KibaLayoutData *d = &kiba_layout_data[position] */
	gint          cap_size;       /**< Dock Cap Size (space between
					   the first/last object and the start/end of the dock) */
	gint          object_space;   /**< Space between objects */
	gint          object_margin;  /**< Space between objects and the screen edge */
	gint          object_size;    /**< Object size, this value may is not exact for objects (scaling) */
	gint          height;         /**< The dock height */
	gint          max_size;       /**< Max size of the dock */
	gint          offset;         /**< Offset of the dock center to the screen center */
};

typedef enum {
	DOCK_UNHIDDEN = 0,
	DOCK_UNHIDING,
	DOCK_HIDDEN,
	DOCK_HIDING
}KibaDockHideState;

typedef struct {
	KibaDockHideState state;  /**< The hide state */
	gint              area;   /**< The hide area */
	gint              speed;  /**< The hide speed */
	gint              delay;  /**< The delay to wait before the hide starts */
}KibaAutoHide;

typedef struct {
	gboolean      draw;  /**< TRUE if the dock should be rendered, otherwise FALSE */
	
	union {
		KibaBackgroundStyle   style;  /**< The background style */
		char                 *file;   /**< The pattern file path */
	} background;
	
	float background_alpha;          /**< Background alpha */
	float bg_affection;              /**< Affection for linear style */
	float bg_step_size;              /**< Step size for radial/linear style */
	float bg_radius_1;               /**< Radius for radial style */
	float bg_radius_2;               /**< Radius for radial style */
	
	float reflection_alpha;          /**< Reflection alpha */
	float reflection_scale_factor;   /**< Reflection scale factor */
	
	KibaRGBA border_rgba;             /**< Border color */
	KibaRGBA prelight_rgba;           /**< Prelight color */
	KibaRGBA curve_color;             /**< Curve color */
	KibaRGBA *gradient;
	gint numColors;
}KibaDockStyle;

typedef struct {
	KibaDock             *dock;       /**< The KibaDock instance */
	KibaDockPopulateFunc  func;       /**< The populate function */
	char                 *icon;       /**< The icon for the menu item */
	char                 *title;      /**< The text for the menu item */
	gpointer              user_data;  /**< The user data */
}KibaDockPopulateData;

struct _KibaDock {
	GtkBox          parent_instance;
	
	gint            flags; /**< object flags */
	
	KibaDockLayout   d;
	float           scroll_offset; /**<< object offset for dock scrolling **/
	KibaDockStyle    style;
	KibaAutoHide     autohide;
	
	CairoSurfaceData surface_data;
	struct _KibaDockShape {
		KibaDockShapeDrawFunc      draw_func;
		KibaDockSeparatorDrawFunc  separator_draw_func;
		KibaDockClipFunc           clip_func;
		gboolean                   triangle_ontop;
		gpointer                   draw_data;
		
		KibaCornerShape            topleft;
		KibaCornerShape            topright;
		KibaCornerShape            bottomright;
		KibaCornerShape            bottomleft;
		float                     corner_radius;
	}shape;
	
	GList          *groups;        /**< List of all object groups */
	GList          *populate_data; /**< List of all added KibaDockPopulateData instances */
	
	KibaDockChildPositionFunc allocate_children; /**< object positioning method */
	gpointer allocate_children_data; /**< user data for the positioning method */
	
	cairo_surface_t *gearSurface; /**< gear surface for the objects, to show there is work in progress **/
	cairo_surface_t *gearSurfaceOverlay; /**< the not moving overlay for the gear surface **/
	struct {
		gboolean enable;
		char *icon;
		gint size;
		float step;
	}continuousProgress;
};

struct _KibaDockClass {
	GtkBoxClass     parent_class;
	
	/* signals */
	void (* position_changed) (KibaDock     *dock,
				   KibaPosition  position);
	void (* orientation_changed) (KibaDock       *dock,
				      GtkOrientation  orientation);
	void (* autohide_toggled) (KibaDock *dock,
				   gboolean  enabled);
	void (* height_changed) (KibaDock *dock,
				      gint      new_height);
	void (* object_size_changed) (KibaDock *dock,
				      gint      new_size);
};

GtkWidget* kiba_dock_new (void);

gboolean kiba_dock_remove_custom_children_position_func_by_data (KibaDock *dock,
								 gpointer  data);

gboolean kiba_dock_set_custom_children_position_func (KibaDock                 *dock,
						      KibaDockChildPositionFunc func,
						      gpointer                  data);

void kiba_dock_add_populate_func (KibaDock             *dock,
				  KibaDockPopulateFunc  func,
				  const char           *icon,
				  const char           *title,
				  gpointer              user_data);

void kiba_dock_remove_populate_func (KibaDock             *dock,
				     KibaDockPopulateFunc  func);

void kiba_dock_set_group_sort_func (KibaDock         *dock,
				    const char       *group,
				    KibaGroupSortFunc func,
				    gpointer          data);

void kiba_dock_remove_group_sort_func (KibaDock    *dock,
				       const char  *group);

gboolean kiba_dock_object_drag_active (KibaDock *dock);

void kiba_dock_set_position (KibaDock     *dock,
			     KibaPosition  position);

KibaPosition kiba_dock_get_position (KibaDock *dock);

void kiba_dock_expose (KibaDock *dock,
		       cairo_t  *cr);

void kiba_dock_paint_object (KibaDock *dock,
                             KibaDrawableObject *drawable_object,
                             cairo_t *cr);

void kiba_dock_update_surface (KibaDock *dock);

void kiba_dock_update_strut (GtkWidget *widget);

G_END_DECLS

#endif
