/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * Simple plugin API.
 **/

#ifndef __KIBA_PLUGIN_H__
#define __KIBA_PLUGIN_H__

#include <gtk/gtk.h>
#include <glib-object.h>
#include <gdk/gdk.h>
#include <glib.h>

#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>

#include <kiba-prefs.h>

G_BEGIN_DECLS

typedef struct _KibaPlugin       KibaPlugin;
typedef struct _KibaPluginClass  KibaPluginClass;
typedef struct _KibaPluginVtable KibaPluginVtable;

#define KIBA_TYPE_PLUGIN           (kiba_plugin_get_type ())
#define KIBA_PLUGIN(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_PLUGIN, KibaPlugin))
#define KIBA_PLUGIN_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_PLUGIN,  KibaPluginClass))
#define KIBA_IS_PLUGIN(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_PLUGIN))
#define KIBA_IS_PLUGIN_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_PLUGIN))
#define KIBA_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_PLUGIN, KibaPluginClass))

#define KIBA_PLUGIN_GLOBAL(plug) kiba_plugin_type(plug) == KIBA_PLUGIN_GLOBAL

#define KIBA_PLUGIN_DBUS_PATH  "/org/kiba/dock/KibaPlugin"
#define KIBA_PLUGIN_DBUS_NAMESPACE  "org.kiba.dock.KibaPlugin"

/**
 * KIBA_INIT_PLUGIN:
 * @vtable: The KibaPluginVtable.
 * @plugin_settings: A PrefMember array.
 * @plugin_services: A KibaPluginServiceData array of services offered by the plugin.
 * @plugin_dependencies: A KibaNeededPlugin array of plugin IDs the plugin depends on.
 *
 * Handles the initialization of modules.
 *
 * Returns: Wether the Plugin was registered succesfull.
 */
#define KIBA_INIT_PLUGIN(vtable, plugin_settings, plugin_services, plugin_dependencies, plugin_conflicts) \
	gboolean kiba_init_plugin(KibaPlugin *plugin);\
	gboolean kiba_init_plugin(KibaPlugin *plugin) { \
		plugin->vtable = &(vtable); \
		plugin->options = plugin_settings; \
		plugin->option_elements = plugin_settings != NULL ? G_N_ELEMENTS(plugin_settings) : 0; \
		plugin->dependencies = plugin_dependencies; \
		plugin->num_dependencies = plugin_dependencies != NULL ? G_N_ELEMENTS(plugin_dependencies) : 0; \
		plugin->conflicts = plugin_conflicts; \
		plugin->num_conflicts = plugin_conflicts != NULL ? G_N_ELEMENTS(plugin_conflicts) : 0; \
		if (plugin_services != NULL) \
			kiba_plugin_set_up_dbus(plugin, plugin_services); \
		return kiba_plugin_register (plugin); \
	}

typedef enum {
	KIBA_PLUGIN_GLOBAL = 0,
	KIBA_PLUGIN_DOCK,
	KIBA_PLUGIN_TYPE_LAST,
}KibaPluginType;

typedef struct {
	char *id;
	KibaPluginType type;
	KibaPrefsFile *file;
}KibaPluginDependencyData;

typedef struct {
	const char *id;
	KibaPluginType type;
}KibaPluginDependency;
typedef KibaPluginDependency KibaPluginConflict;

typedef enum {
	KIBA_PLUGIN_MULTIPLE_LOADABLE = 0,
	KIBA_PLUGIN_ONCE_LOADABLE = 1
}KibaPluginLoadableFlag;

/**
 * KibaPluginVtable:
 *
 * A Plugin VTable.
 */
struct _KibaPluginVtable {
	const char *summary;      /**< plugin summary        */
	const char *description;  /**< plugin description    */
	
	const char *author;       /**< plugin author         */
	const char *homepage;     /**< plugin homepage       */
	
	KibaPluginLoadableFlag loadable_flag; /**< flag indicating if the plugin can be loaded multiple times */
	
	gboolean (*load) (KibaPlugin *plugin); /**< plugin load function */
	void  (*unload) (KibaPlugin *plugin);  /**< plugin unload function */
};

/**
 * A Plugin Handle.
 */
struct _KibaPlugin {
	GTypeModule parent_instance;
	
	KibaPluginVtable *vtable; /**< the KibaPluginVtable */
	
	gboolean enabled; /**< represents $(plugin_name)_enable from Prefs */
	gboolean loaded; /**< TRUE if Plugin is loaded */
	
	PrefMember *options; /**< array of all plugin options */
	gint option_elements;  /**< num of array member */
	
	KibaPluginDependency *dependencies; /**< array of all plugin dependencies */
	gint num_dependencies;  /**< number of plugin dependencies */
	
	KibaPluginConflict *conflicts; /**< array of all plugin dependencies */
	gint num_conflicts;  /**< number of plugin dependencies */
	
	const DBusGObjectInfo *dbus_info; /**< dbus/service info */
};

struct _KibaPluginClass {
	GTypeModuleClass parent_class;
	
	/* signals */
	void (* unloaded) (KibaPlugin *plugin);
	void (* loaded) (KibaPlugin *plugin);
	void (*dbus_registered) (KibaPlugin *plugin);
};

KibaPlugin* kiba_plugin_new (const char *library_path,
                             KibaPluginType type);

gboolean kiba_plugin_register (KibaPlugin *plugin);

KibaPluginType kiba_plugin_type(KibaPlugin *plugin);

const char* kiba_plugin_get_lib_path(KibaPlugin *plugin);

void kiba_plugin_set_up_dbus (KibaPlugin *plugin,
                              const DBusGObjectInfo *info);

G_END_DECLS

#endif
